/*
 * (c) 2002 Jaeger Computergesteuerte Messtechnik GmbH, Lorsch
 *
 *	v2.0	Author: Daniel Franke <info@ADwin.de>
 *		Date:	06/30/2002
 *
 * 	v1.0	Author: Marcus Galemann <info@ADwin.de>
 *
 * ChangeLog:
 *	v2.0
 */

#ifndef __ADWIN_H__
#define __ADWIN_H__

#include <linux/config.h>

#define MODULE_NAME "adwin"

#if defined(__KERNEL__) && !defined(CONFIG_PROC_FS)
	#error
	#error No procfs found. This is necessary for usage of this module!
	#error Please reconfigure and rebuild your kernel accordingly,
	#error then try again.
	#error
#endif

#define ADWIN_MAJOR 60

/*
 * DP - Device Processor
 */
#define DP_UNKNOWN	0x0
#define DP_T2		0x1
#define DP_T4		0x2
#define DP_T8		0x4
#define DP_DSP		0x100		/* ADSP 2106 (40 MHz) */
#define DP_21160	0x200		/* xxx (80 MHz) */

/*
 * DC - Device Connection
 * ACHTUNG: DC_USB und DC_ETHERNET nur in Verbindung mit DP_DSP !
 */
#define DC_UNKNOWN	0x00000
#define DC_LINK	 	0x10000
#define DC_PCMCIA	0x30000		/* PCMCIA == LINK */
#define DC_USB		0x40000
#define DC_ETHERNET	0x80000

/*
 * Bitmask: type / processor
 *
 *         3          2          1
 *	 21098765 43210987 65432109 87654321
 *	+--------+--------+--------+--------+
 *	|        |    NUPL|      2D|     TTT|
 *	|        |    ESCI|      1S|     842|
 *	|        |    TBMN|      1P|        |
 *	|        |      CK|      6 |        |
 *	|        |      I |      0 |        |
 *	|        |      A |        |        |
 *	+--------+--------+--------+--------+
 */

/*
 * device-structure
 * used to store the device-info, the library uses ioctl(GET_DEV_STRUCT)
 * to get a copy (structure is stored in data-member of struct proc_dir_entry)
 *
 * The structures adwin_dev_t und adwin_kdev_t differ only in a few members
 * at the end of kdev. We store kdev-structures and cast them to dev-structures
 * if the user requests them.
 */

/* include target dependent */
#ifdef __KERNEL__
	#include <linux/in.h>
	#include <asm/atomic.h>
	#include <linux/wait.h>
	#include <linux/proc_fs.h>
#else
	#include <netinet/in.h>
	#include <sys/types.h>
#endif

#include <linux/types.h>

#define DESCLEN	32
#define PWLEN	10
#define MAX_DEVICES 100

typedef struct {
	uid_t dev_uid;
	gid_t dev_gid;
	mode_t dev_mode;

	int dev_name;
	char dev_desc[DESCLEN];
	int dev_type;				/* connection type */

	int dev_link_base_addr;

 	int dev_usb_serial;

	int dev_net_host_link;			/* device ID at IP */
	char  dev_net_host_password[PWLEN];
	int dev_net_package_count;		/* default: 5 */
	int dev_net_timeout;			/* default: 1000ms */

	struct sockaddr_in dev_net_server;

	int dev_processor;
} adwin_dev_t;

/* write 6 opcode-bytes linear to dsp */
#define ADIOSOPCODE		_IOR(0xAD, 107, char[6])

/*
 * ioctl() with /proc/adwin/dev/0x*
 */
#define ADIORESETLINK		_IO(0xAD, 104)
#define ADIOGREADSTATUS		_IO(0xAD, 105)
#define ADIOGWRITESTATUS	_IO(0xAD, 106)
#define ADIOSKDEVPROC		_IOW(0xAD, 107, int)

#define ADIOGKDEVINFO		_IOR(0xAD, 100, adwin_dev_t)
#define ADIOSKDEVINFO		_IOW(0xAD, 100, adwin_dev_t)

#ifdef __KERNEL__
typedef struct {
	adwin_dev_t dev;

	atomic_t kdev_in_use;
	atomic_t kdev_use_count;
	wait_queue_head_t kdev_queue;
} adwin_kdev_t;

#endif

#define kdev_name		dev.dev_name
#define kdev_uid		dev.dev_uid
#define kdev_gid		dev.dev_gid
#define kdev_mode		dev.dev_mode
#define kdev_desc		dev.dev_desc
#define kdev_type		dev.dev_type
#define kdev_link_base_addr	dev.dev_link_base_addr
#define kdev_usb_serial		dev.dev_usb_serial
#define kdev_net_host_link	dev.dev_net_host_link
#define kdev_net_host_pasword	dev.dev_net_host_password
#define kdev_net_package_count	dev.dev_net_package_count
#define kdev_net_timeout	dev.dev_net_timeout
#define kdev_net_server		dev.dev_net_server
#define kdev_net_host_password	dev.dev_net_host_password
#define kdev_processor		dev.dev_processor


/*
 * ioctl() with /proc/adwin/dev/0x*
 */
#define ADIOGKDEVDESC		_IOR(0xAD,  2, char)
#define ADIOSKDEVDESC		_IOW(0xAD,  2, char)
#define ADIOGKDEVUID		_IOR(0xAD,  3, uid_t)
#define ADIOSKDEVUID		_IOW(0xAD,  3, uid_t)
#define ADIOGKDEVGID		_IOR(0xAD,  4, gid_t)
#define ADIOSKDEVGID		_IOW(0xAD,  4, gid_t)
#define ADIOGKDEVMODE		_IOR(0xAD,  5, mode_t)
#define ADIOSKDEVMODE		_IOW(0xAD,  5, mode_t)

#define ADIOGKDEVTYPE		_IOR(0xAD,  6, int)
#define ADIOSKDEVTYPE		_IOW(0xAD,  6, int)
#define ADIOGKDEVBASE		_IOR(0xAD,  7, int)
#define ADIOSKDEVBASE		_IOW(0xAD,  7, int)
#define ADIOGKDEVSERIAL		_IOR(0xAD,  8, int)
#define ADIOSKDEVSERIAL		_IOW(0xAD,  8, int)
#define ADIOGKDEVSERVER		_IOR(0xAD,  9, struct sockaddr_in)
#define ADIOSKDEVSERVER		_IOW(0xAD,  9, struct sockaddr_in)

#define ADIOGKDEVTIMEO		_IOR(0xAD, 10, int)
#define ADIOSKDEVTIMEO		_IOW(0xAD, 10, int)
#define ADIOGKDEVCOUNT		_IOR(0xAD, 11, int)
#define ADIOSKDEVCOUNT		_IOW(0xAD, 11, int)
#define ADIOGKDEVHSTLNK		_IOR(0xAD, 12, int)
#define ADIOSKDEVHSTLNK		_IOW(0xAD, 12, int)
#define ADIOGKDEVPW		_IOR(0xAD, 13, char)
#define ADIOSKDEVPW		_IOW(0xAD, 13, char)



/*
 * ioctl() with /proc/adwin/config
 */
#define ADIOKDEVADD		_IOW(0xAD, 101, int)
#define ADIOKDEVDEL		_IOW(0xAD, 102, int)



#endif /* __DEVICEADWIN_H__ */
