/*
 Die Case - Makros sind dazu gedacht, in eine switch (dev.dev_type | dev.dev_processor) - Anweisung
 eingebaut zu werden. Sie bilden einen Funktionsnamen wie "DSP_Get_Par" und so weiter.
 Fr die Ethernet-Version gibt es zwei verschiedene Versionen, je nachdem, ob ein Rckgabewert zurckgeliefert werden
 soll oder nicht, denn die Funktionen des generischen Treibers liefern, anders als die anderen, keine
 Fehlercodes zurck.
 hnlich die SELECT_PROTOCOL - Makros, die jeweils auf die CASE-Makros zurckgreifen - die Zahl hinter dem Namen
 gibt die Anzahl der zu bergebenden Parameter an. */

#if defined(WITH_ETHERNET)
	#define CASE_NET_NO_RET(retval, func, args...)		\
		case DP_DSP | DC_ETHERNET:			\
		case DP_UNKNOWN | DC_ETHERNET:			\
		{						\
			Generic_Driver_##func(args);		\
			adwin_errno = Generic_Driver_Get_Last_Error ();	\
			if (adwin_errno)			\
				retval = -1;			\
			else					\
				retval = 0;			\
			break;					\
		}
// -------------------------------------------------------------------------
	#define CASE_NET_RET(retval, func, args...)		\
		case DP_DSP | DC_ETHERNET:			\
		case DP_UNKNOWN | DC_ETHERNET:			\
		{						\
			retval = Generic_Driver_##func(args);	\
			adwin_errno = Generic_Driver_Get_Last_Error ();	\
			break;					\
		}
// ----------------------------------------------------------------------------
#else
	#define CASE_NET_NO_RET(func, args...)
	#define CASE_NET_RET(func, args...)
#endif

#if defined(WITH_LINK)
	#define CASE_DSP(retval, func, args...)		\
		case DP_DSP | DC_LINK:			\
			retval = DSP_##func(args);	\
			break;

	#define CASE_TRANSPUTER(retval, func, args...)	\
		case DP_T2 | DC_LINK:			\
		case DP_T4 | DC_LINK:			\
		case DP_T8 | DC_LINK:			\
			retval = T_##func(args);	\
			break;
#else
	#define CASE_DSP(retval, func, args...)
	#define CASE_TRANSPUTER(retval, func, args...)
#endif

#if defined(WITH_USB)
	#define CASE_USB(retval, func, args...)		\
		case DP_DSP | DC_USB:			\
		case DP_UNKNOWN | DC_USB:		\
			retval = USB_##func(args);	\
			break;
#else
	#define CASE_USB(retval, func, args...)
#endif

/* Falls eine Device ein Ethernet-Device ist, mu vorher mit einem
*  Set_DeviceNo der generische Treiber initialisiert worden sein.
*  Deshalb braucht man den NET_-Funktionen nicht mehr dir adwin_dev_t- und fd-Parameter bergeben
*/

#define SELECT_PROTOCOL(retval, func, dev, fd, args) 	\
	switch((dev)->dev_processor | (dev)->dev_type){	\
		CASE_NET_NO_RET(retval, func, args)	\
		CASE_DSP(retval, func, dev, fd, args)	\
		CASE_USB(retval, func, dev, fd, args)	\
		CASE_TRANSPUTER(retval, func, dev, fd, args)\
		default:				\
			errno = EINVAL;			\
			adwin_errno = -EINVAL;		\
			retval = -1;			\
	}

#define SELECT_PROTOCOL_WITH_RETURN0(retval, func, dev, fd) 	\
	switch((dev)->dev_processor | (dev)->dev_type){	\
		CASE_NET_RET(retval, func)	\
		CASE_DSP(retval, func, dev, fd)	\
		CASE_USB(retval, func, dev, fd)	\
		CASE_TRANSPUTER(retval, func, dev, fd)\
		default:				\
			errno = EINVAL;			\
			adwin_errno = -EINVAL;		\
			retval = -1;			\
	}


#define SELECT_PROTOCOL_WITH_RETURN1(retval, func, dev, fd, arg1, pdest) 	\
	switch((dev)->dev_processor | (dev)->dev_type){	\
		CASE_NET_RET(*pdest, func, arg1)	\
		CASE_DSP(retval, func, dev, fd, arg1, pdest)	\
		CASE_USB(retval, func, dev, fd, arg1, pdest)	\
		CASE_TRANSPUTER(retval, func, dev, fd, arg1, pdest)\
		default:				\
			errno = EINVAL;			\
			adwin_errno = -EINVAL;		\
			retval = -1;			\
	}

#define SELECT_PROTOCOL_WITH_RETURN2(retval, func, dev, fd, arg1, arg2, pdest)	\
	switch((dev)->dev_processor | (dev)->dev_type){	\
		CASE_NET_NO_RET(retval, func, pdest, arg1, arg2)	\
		CASE_DSP(retval, func, dev, fd, arg1, arg2, pdest)	\
		CASE_USB(retval, func, dev, fd, arg1, arg2, pdest)	\
		CASE_TRANSPUTER(retval, func, dev, fd, arg1, arg2, pdest)\
		default:				\
			errno = EINVAL;			\
			adwin_errno = -EINVAL;		\
			retval = -1;			\
	}

#define SELECT_PROTOCOL_DATA(GetSet, retval, dev, fd, dataID, first, count, dsp_type, void_ptr)     \
	switch((dev)->dev_processor | (dev)->dev_type){	\
		CASE_NET_NO_RET(retval, GetSet##_Data, dataID, void_ptr, dsp_type == 3 ? 2 : dsp_type, first, count)	\
		CASE_DSP(retval, GetSet##Data, dev, fd, dataID, first, count, void_ptr)	\
		CASE_USB(retval, GetSet##Data, dev, fd, dataID, first, count, void_ptr)	\
		CASE_TRANSPUTER(retval, GetSet##Data, dev, fd, dataID, first, count, void_ptr) \
		default:				\
			errno = EINVAL;			\
			adwin_errno = -EINVAL;		\
			retval = -1;			\
	}

#define SELECT_PROTOCOL_FIFO(GetSet, retval, dev, fd, dataID, count, dsp_type, void_ptr)     \
	switch((dev)->dev_processor | (dev)->dev_type){	\
		CASE_NET_NO_RET(retval, GetSet##_Fifo, dataID, dsp_type, void_ptr, count)	\
		CASE_DSP(retval, GetSet##Fifo, dev, fd, dataID, count, void_ptr)	\
		CASE_USB(retval, GetSet##Fifo, dev, fd, dataID, count, void_ptr)	\
		CASE_TRANSPUTER(retval, GetSet##Fifo, dev, fd, dataID, count, void_ptr) \
		default:				\
			errno = EINVAL;			\
			adwin_errno = -EINVAL;		\
			retval = -1;			\
	}
